@@ -1,4 +1,5 @@
 import numpy as np
+from ogcore import aggregates as aggr
 
 """
 ------------------------------------------------------------------------
@@ -95,12 +96,7 @@ def get_Y(K, K_g, L, p, method, m=-1):
                     * (L ** ((epsilon - 1) / epsilon))
                 )
             ) ** (epsilon / (epsilon - 1))
-            Y2 = (
-                Z
-                * (K**gamma)
-                * (K_g**gamma_g)
-                * (L ** (1 - gamma - gamma_g))
-            )
+            Y2 = Z * (K**gamma) * (K_g**gamma_g) * (L ** (1 - gamma - gamma_g))
             Y[epsilon == 1] = Y2[epsilon == 1]
     else:  # TPI case
         if m is not None:
@@ -156,12 +152,7 @@ def get_Y(K, K_g, L, p, method, m=-1):
                     * (L ** ((epsilon - 1) / epsilon))
                 )
             ) ** (epsilon / (epsilon - 1))
-            Y2 = (
-                Z
-                * (K**gamma)
-                * (K_g**gamma_g)
-                * (L ** (1 - gamma - gamma_g))
-            )
+            Y2 = Z * (K**gamma) * (K_g**gamma_g) * (L ** (1 - gamma - gamma_g))
             Y[:, epsilon == 1] = Y2[:, epsilon == 1]
 
     return Y
@@ -462,9 +453,7 @@ def get_L_from_Y(w, Y, p, method):
         Z = p.Z[-1]
     else:
         Z = p.Z[: p.T]
-    L = ((1 - p.gamma - p.gamma_g) * Z ** (p.epsilon - 1) * Y) / (
-        w**p.epsilon
-    )
+    L = ((1 - p.gamma - p.gamma_g) * Z ** (p.epsilon - 1) * Y) / (w**p.epsilon)
 
     return L
 
@@ -665,9 +654,7 @@ def solve_L(Y, K, K_g, p, method, m=-1):
             K_g[K_g == 0] = 1.0
             gamma_g = 0
     if epsilon == 1.0:
-        L = (Y / (Z * K**gamma * K_g**gamma_g)) ** (
-            1 / (1 - gamma - gamma_g)
-        )
+        L = (Y / (Z * K**gamma * K_g**gamma_g)) ** (1 / (1 - gamma - gamma_g))
     else:
         L = (
             (
@@ -679,3 +666,30 @@ def solve_L(Y, K, K_g, p, method, m=-1):
         ) ** (epsilon / (epsilon - 1))
 
     return L
+
+
+def adj_cost(K, Kp1, p, method):
+    r"""
+    Firm capital adjstment costs
+
+    ..math::
+        \Psi(K_{t}, K_{t+1}) = \frac{\psi}{2}\biggr(\frac{\biggr(\frac{I_{t}}{K_{t}}-\mu\biggl)^{2}}{\frac{I_{t}}{K_{t}}}\biggl)
+
+    Args:
+        K (array-like): Current period capital stock
+        Kp1 (array-like): One-period ahead capital stock
+        p (OG-USA Parameters class object): Model parameters
+        method (str): 'SS' or 'TPI'
+
+    Returns
+        Psi (array-like): Capital adjustment costs per unit of investment
+    """
+    if method == "SS":
+        ac_method = "total_ss"
+    else:
+        ac_method = "total_tpi"
+    Inv = aggr.get_I(None, Kp1, K, p, ac_method)
+
+    Psi = ((p.psi / 2) * (Inv / K - p.mu) ** 2) / (Inv / K)
+
+    return Psi
