@@ -3,69 +3,123 @@ import pandas as pd
 import os
 import matplotlib.pyplot as plt
 import matplotlib
+from cycler import cycler
 from ogcore.constants import GROUP_LABELS
 from ogcore import utils, txfunc
 from ogcore.constants import DEFAULT_START_YEAR, VAR_LABELS
 
 
-def plot_imm_rates(p, year=DEFAULT_START_YEAR, include_title=False, path=None):
+def plot_imm_rates(
+    imm_rates,
+    start_year=DEFAULT_START_YEAR,
+    years_to_plot=[DEFAULT_START_YEAR],
+    include_title=False,
+    source="United Nations, World Population Prospects",
+    path=None,
+):
     """
-    Create a plot of immigration rates from OG-Core parameterization.
+    Plot fertility rates from the data
 
     Args:
-        p (OG-Core Specifications class): parameters object
-        year (integer): year of mortality ratese to plot
-        include_title (bool): whether to include a title in the plot
-        path (string): path to save figure to
+        imm_rates (NumPy array): immigration rates for each of
+            totpers
+        start_year (int): first year of data
+        years_to_plot (list): list of years to plot
+        source (str): data source for fertility rates
+        path (str): path to save figure to, if None then figure
+            is returned
 
     Returns:
-        fig (Matplotlib plot object): plot of immigration rates
+        fig (Matplotlib plot object): plot of fertility rates
 
     """
-    assert isinstance(year, int)
-    age_per = np.linspace(p.E, p.E + p.S, p.S)
+    # create line styles to cycle through
+    plt.rc("axes", prop_cycle=(cycler("linestyle", [":", "-.", "-", "--"])))
     fig, ax = plt.subplots()
-    plt.scatter(age_per, p.imm_rates[year - p.start_year, :], s=40, marker="d")
-    plt.plot(age_per, p.imm_rates[year - p.start_year, :])
-    plt.xlabel(r"Age $s$ (model periods)")
-    plt.ylabel(r"Imm. rate $i_{s}$")
-    vals = ax.get_yticks()
-    ax.set_yticklabels(["{:,.2%}".format(x) for x in vals])
+    for y in years_to_plot:
+        i = start_year - y
+        plt.plot(imm_rates[i, :], c="blue", label="Year " + str(y))
+    # plt.title('Fertility rates by age ($f_{s}$)',
+    #     fontsize=20)
+    plt.xlabel(r"Age $s$")
+    plt.ylabel(r"Immigration rate $i_{s}$")
+    plt.legend(loc="upper left")
+    plt.text(
+        -5,
+        -0.023,
+        "Source: " + source,
+        fontsize=9,
+    )
+    plt.tight_layout(rect=(0, 0.035, 1, 1))
     if include_title:
-        plt.title("Immigration Rates in " + str(year))
-    if path is None:
-        return fig
+        plt.title("Immigration Rates")
+    # Save or return figure
+    if path:
+        output_path = os.path.join(path, "imm_rates")
+        plt.savefig(output_path, dpi=300)
+        plt.close()
     else:
-        fig_path = os.path.join(path, "imm_rates_orig")
-        plt.savefig(fig_path, dpi=300)
+        fig.show()
+        return fig
 
 
-def plot_mort_rates(p, include_title=False, path=None):
+def plot_mort_rates(
+    p_list,
+    labels=[""],
+    years=[DEFAULT_START_YEAR],
+    survival_rates=False,
+    include_title=False,
+    path=None,
+):
     """
     Create a plot of mortality rates from OG-Core parameterization.
 
     Args:
-        p (OG-Core Specifications class): parameters object
+        p_list (list): list of parameters objects
+        labels (list): list of labels for the legend
+        survival_rates (bool): whether to plot survival rates instead
+            of mortality rates
         include_title (bool): whether to include a title in the plot
         path (string): path to save figure to
 
     Returns:
-        fig (Matplotlib plot object): plot of immigration rates
+        fig (Matplotlib plot object): plot of mortality rates
 
     """
-    age_per = np.linspace(p.E, p.E + p.S, p.S)
+    p0 = p_list[0]
+    age_per = np.linspace(p0.E, p0.E + p0.S, p0.S)
     fig, ax = plt.subplots()
-    plt.plot(age_per, p.rho)
+    for y in years:
+        t = y - p0.start_year
+        for i, p in enumerate(p_list):
+            if survival_rates:
+                plt.plot(
+                    age_per,
+                    np.cumprod(1 - p.rho[t, :]),
+                    label=labels[i] + " " + str(y),
+                )
+            else:
+                plt.plot(age_per, p.rho[t, :], label=labels[i] + " " + str(y))
     plt.xlabel(r"Age $s$ (model periods)")
-    plt.ylabel(r"Mortality Rates $\rho_{s}$")
+    if survival_rates:
+        plt.ylabel(r"Cumulative Survival Rates")
+        plt.legend(loc="lower left")
+        title = "Survival Rates"
+    else:
+        plt.ylabel(r"Mortality Rates $\rho_{s}$")
+        plt.legend(loc="upper left")
+        title = "Mortality Rates"
     vals = ax.get_yticks()
     ax.set_yticklabels(["{:,.0%}".format(x) for x in vals])
     if include_title:
-        plt.title("Mortality Rates")
+        plt.title(title)
     if path is None:
         return fig
     else:
-        fig_path = os.path.join(path, "mortality_rates")
+        if survival_rates:
+            fig_path = os.path.join(path, "survival_rates")
+        else:
+            fig_path = os.path.join(path, "mortality_rates")
         plt.savefig(fig_path, dpi=300)
 
 
@@ -148,27 +202,56 @@ def plot_population(p, years_to_plot=["SS"], include_title=False, path=None):
         plt.savefig(fig_path, dpi=300)
 
 
-def plot_ability_profiles(p, include_title=False, path=None):
+def plot_ability_profiles(
+    p, p2=None, t=None, log_scale=False, include_title=False, path=None
+):
     """
     Create a plot of earnings ability profiles.
 
     Args:
         p (OG-Core Specifications class): parameters object
+        t (int): model period for year, if None, then plot ability matrix for SS
+        log_scale (bool): whether to plot in log points
+        include_title (bool): whether to include a title in the plot
         path (string): path to save figure to
 
     Returns:
         fig (Matplotlib plot object): plot of earnings ability profiles
 
     """
+    if t is None:
+        t = -1
     age_vec = np.arange(p.starting_age, p.starting_age + p.S)
     fig, ax = plt.subplots()
     cm = plt.get_cmap("coolwarm")
     ax.set_prop_cycle(color=[cm(1.0 * i / p.J) for i in range(p.J)])
     for j in range(p.J):
-        plt.plot(age_vec, p.e[:, j], label=GROUP_LABELS[p.J][j])
+        if log_scale:
+            plt.plot(age_vec, np.log(p.e[t, :, j]), label=GROUP_LABELS[p.J][j])
+        else:
+            plt.plot(age_vec, p.e[t, :, j], label=GROUP_LABELS[p.J][j])
+    if p2 is not None:
+        for j in range(p.J):
+            if log_scale:
+                plt.plot(
+                    age_vec,
+                    np.log(p2.e[t, :, j]),
+                    linestyle="--",
+                    label=GROUP_LABELS[p.J][j],
+                )
+            else:
+                plt.plot(
+                    age_vec,
+                    p2.e[t, :, j],
+                    linestyle="--",
+                    label=GROUP_LABELS[p.J][j],
+                )
     plt.xlabel(r"Age")
-    plt.ylabel(r"Earnings ability")
-    plt.legend(loc=9, bbox_to_anchor=(0.5, -0.15), ncol=2)
+    if log_scale:
+        plt.ylabel(r"ln(Earnings ability)")
+    else:
+        plt.ylabel(r"Earnings ability")
+    plt.legend(loc=9, bbox_to_anchor=(0.5, -0.15), ncols=5)
     if include_title:
         plt.title("Lifecycle Profiles of Effective Labor Units")
     if path is None:
@@ -236,21 +319,37 @@ def plot_elliptical_u(p, plot_MU=True, include_title=False, path=None):
         plt.savefig(fig_path, dpi=300)
 
 
-def plot_chi_n(p, include_title=False, path=None):
+def plot_chi_n(
+    p_list,
+    labels=[""],
+    years_to_plot=[DEFAULT_START_YEAR],
+    include_title=False,
+    path=None,
+):
     """
     Create a plot of showing the values of the chi_n parameters.
 
     Args:
-        p (OG-Core Specifications class): parameters object
+        p_list (list): parameters objects
+        labels (list): labels for legend
+        years_to_plot (list): list of years to plot
+        include_title (boolean): whether to include a title in the plot
         path (string): path to save figure to
 
     Returns:
         fig (Matplotlib plot object): plot of chi_n parameters
 
     """
-    age = np.linspace(p.starting_age, p.ending_age, p.S)
+    p0 = p_list[0]
+    age = np.linspace(p0.starting_age, p0.ending_age, p0.S)
     fig, ax = plt.subplots()
-    plt.plot(age, p.chi_n)
+    for y in years_to_plot:
+        for i, p in enumerate(p_list):
+            plt.plot(
+                age,
+                p.chi_n[y - p.start_year, :],
+                label=labels[i] + " " + str(y),
+            )
     if include_title:
         plt.title("Utility Weight on the Disutility of Labor Supply")
     plt.xlabel("Age, $s$")
@@ -263,163 +362,146 @@ def plot_chi_n(p, include_title=False, path=None):
 
 
 def plot_fert_rates(
-    fert_func,
-    age_midp,
-    totpers,
-    min_yr,
-    max_yr,
-    fert_data,
-    fert_rates,
-    output_dir=None,
+    fert_rates_list,
+    labels=[""],
+    start_year=DEFAULT_START_YEAR,
+    years_to_plot=[DEFAULT_START_YEAR],
+    include_title=False,
+    source="United Nations, World Population Prospects",
+    path=None,
 ):
     """
-    Plot fertility rates from the data along with smoothed function to
-    use for model fertility rates.
+    Plot fertility rates from the data
 
     Args:
-        fert_func (Scipy interpolation object): interpolated fertility
-            rates
-        age_midp (NumPy array): midpoint of age for each age group in
-            data
-        totpers (int): total number of agent life periods (E+S), >= 3
-        min_yr (int): age in years at which agents are born, >= 0
-        max_yr (int): age in years at which agents die with certainty,
-            >= 4
-        fert_data (NumPy array): fertility rates by age group from data
-        fert_rates (NumPy array): fitted fertility rates for each of
-            totpers
-        output_dir (str): path to save figure to, if None then figure
+        fert_rates_list (list): list of Numpy arrays of fertility rates
+            for each model period and age
+        labels (list): list of labels for the legend
+        start_year (int): first year of data
+        years_to_plot (list): list of years to plot
+        include_title (bool): whether to include a title in the plot
+        source (str): data source for fertility rates
+        path (str): path to save figure to, if None then figure
             is returned
 
     Returns:
         fig (Matplotlib plot object): plot of fertility rates
 
     """
-    # Generate finer age vector and fertility rate vector for
-    # graphing cubic spline interpolating function
-    age_fine_pred = np.linspace(age_midp[0], age_midp[-1], 300)
-    fert_fine_pred = fert_func(age_fine_pred)
-    age_fine = np.hstack((min_yr, age_fine_pred, max_yr))
-    fert_fine = np.hstack((0, fert_fine_pred, 0))
-    age_mid_new = np.linspace(
-        np.float64(max_yr) / totpers, max_yr, totpers
-    ) - (0.5 * np.float64(max_yr) / totpers)
-
+    # create line styles to cycle through
+    plt.rc("axes", prop_cycle=(cycler("linestyle", [":", "-.", "-", "--"])))
     fig, ax = plt.subplots()
-    plt.scatter(age_midp, fert_data, s=70, c="blue", marker="o", label="Data")
-    plt.scatter(
-        age_mid_new,
-        fert_rates,
-        s=40,
-        c="red",
-        marker="d",
-        label="Model period (integrated)",
-    )
-    plt.plot(age_fine, fert_fine, label="Cubic spline")
-    # plt.title('Fitted fertility rate function by age ($f_{s}$)',
-    #     fontsize=20)
+    for y in years_to_plot:
+        i = start_year - y
+        for i, fert_rates in enumerate(fert_rates_list):
+            plt.plot(fert_rates[i, :], label=labels[i] + " " + str(y))
+    if include_title:
+        plt.title("Fertility rates by age ($f_{s}$)", fontsize=20)
     plt.xlabel(r"Age $s$")
     plt.ylabel(r"Fertility rate $f_{s}$")
     plt.legend(loc="upper right")
     plt.text(
         -5,
         -0.023,
-        "Source: National Vital Statistics Reports, "
-        + "Volume 64, Number 1, January 15, 2015.",
+        "Source: " + source,
         fontsize=9,
     )
     plt.tight_layout(rect=(0, 0.035, 1, 1))
     # Save or return figure
-    if output_dir:
-        output_path = os.path.join(output_dir, "fert_rates")
+    if path:
+        output_path = os.path.join(path, "fert_rates")
         plt.savefig(output_path, dpi=300)
         plt.close()
     else:
+        fig.show()
         return fig
 
 
 def plot_mort_rates_data(
-    totpers,
-    min_yr,
-    max_yr,
-    age_year_all,
-    mort_rates_all,
-    infmort_rate,
     mort_rates,
-    output_dir=None,
+    start_year=DEFAULT_START_YEAR,
+    years_to_plot=[DEFAULT_START_YEAR],
+    source="United Nations, World Population Prospects",
+    path=None,
 ):
     """
-    Plots mortality rates from the model and data.
+    Plots mortality rates from the data.
 
     Args:
-        totpers (int): total number of agent life periods (E+S), >= 3
-        min_yr (int): age in years at which agents are born, >= 0
-        max_yr (int): age in years at which agents die with certainty,
-            >= 4
-        age_year_all (array_like): ages in mortality rate data
-        mort_rates_all (array_like): mortality rates by age from data,
-            average across males and females
-        infmort_rate (scalar): infant mortality rate
-        mort_rates (array_like): fitted mortality rates for each of
+        mort_rates (array_like): mortality rates for each of
             totpers
-        output_dir (str): path to save figure to, if None then figure
+        start_year (int): first year of data
+        years_to_plot (list): list of years to plot
+        source (str): data source for fertility rates
+        path (str): path to save figure to, if None then figure
             is returned
 
     Returns:
         fig (Matplotlib plot object): plot of mortality rates
 
     """
-    age_mid_new = np.linspace(
-        np.float64(max_yr) / totpers, max_yr, totpers
-    ) - (0.5 * np.float64(max_yr) / totpers)
+    # create line styles to cycle through
+    plt.rc("axes", prop_cycle=(cycler("linestyle", [":", "-.", "-", "--"])))
     fig, ax = plt.subplots()
-    plt.scatter(
-        np.hstack([0, age_year_all]),
-        np.hstack([infmort_rate, mort_rates_all]),
-        s=20,
-        c="blue",
-        marker="o",
-        label="Data",
-    )
-    plt.scatter(
-        np.hstack([0, age_mid_new]),
-        np.hstack([infmort_rate, mort_rates]),
-        s=40,
-        c="red",
-        marker="d",
-        label="Model period (cumulative)",
-    )
-    plt.plot(
-        np.hstack([0, age_year_all[min_yr - 1 : max_yr]]),
-        np.hstack([infmort_rate, mort_rates_all[min_yr - 1 : max_yr]]),
-    )
-    plt.axvline(x=max_yr, color="red", linestyle="-", linewidth=1)
-    plt.grid(visible=True, which="major", color="0.65", linestyle="-")
-    # plt.title('Fitted mortality rate function by age ($rho_{s}$)',
+    for y in years_to_plot:
+        i = start_year - y
+        plt.plot(mort_rates[i, :], c="blue", label="Year " + str(y))
+    # plt.title('Fertility rates by age ($f_{s}$)',
     #     fontsize=20)
     plt.xlabel(r"Age $s$")
-    plt.ylabel(r"Mortality rate $\rho_{s}$")
+    plt.ylabel(r"Mortality rate $rho_{s}$")
     plt.legend(loc="upper left")
     plt.text(
         -5,
-        -0.2,
-        "Source: Actuarial Life table, 2011 Social Security "
-        + "Administration.",
+        -0.223,
+        "Source: " + source,
         fontsize=9,
     )
-    plt.tight_layout(rect=(0, 0.03, 1, 1))
+    plt.tight_layout(rect=(0, 0.035, 1, 1))
     # Save or return figure
-    if output_dir:
-        output_path = os.path.join(output_dir, "mort_rates")
+    if path:
+        output_path = os.path.join(path, "mort_rates")
         plt.savefig(output_path, dpi=300)
         plt.close()
     else:
+        fig.show()
         return fig
 
 
-def plot_omega_fixed(
-    age_per_EpS, omega_SS_orig, omega_SSfx, E, S, output_dir=None
-):
+def plot_g_n(p_list, label_list=[""], include_title=False, path=None):
+    """
+    Create a plot of population growth rates from OG-Core parameterization.
+
+    Args:
+        p_list (list): list of OG-Core Specifications objects
+        label_list (list): list of labels for the legend
+        include_title (bool): whether to include a title in the plot
+        path (string): path to save figure to
+
+    Returns:
+        fig (Matplotlib plot object): plot of immigration rates
+
+    """
+    p0 = p_list[0]
+    years = np.arange(p0.start_year, p0.start_year + p0.T)
+    fig, ax = plt.subplots()
+    for i, p in enumerate(p_list):
+        plt.plot(years, p.g_n[: p.T], label=label_list[i])
+    plt.xlabel(r"Year $s$ (model periods)")
+    plt.ylabel(r"Population Growth Rate $g_{n,t}$")
+    plt.legend(loc="upper right")
+    vals = ax.get_yticks()
+    ax.set_yticklabels(["{:,.0%}".format(x) for x in vals])
+    if include_title:
+        plt.title("Population Growth Rates")
+    if path is None:
+        return fig
+    else:
+        fig_path = os.path.join(path, "pop_growth_rates")
+        plt.savefig(fig_path, dpi=300)
+
+
+def plot_omega_fixed(age_per_EpS, omega_SS_orig, omega_SSfx, E, S, path=None):
     """
     Plot the steady-state population distribution implied by the data
     on fertility and mortality rates versus the the steady-state
@@ -436,7 +518,7 @@ def plot_omega_fixed(
             after adjustment to immigration rates
         E (int): age at which household becomes economically active
         S (int): number of years which household is economically active
-        output_dir (str): path to save figure to, if None then figure
+        path (str): path to save figure to, if None then figure
             is returned
 
     Returns:
@@ -453,8 +535,8 @@ def plot_omega_fixed(
     plt.xlim((0, E + S + 1))
     plt.legend(loc="upper right")
     # Save or return figure
-    if output_dir:
-        output_path = os.path.join(output_dir, "OrigVsFixSSpop")
+    if path:
+        output_path = os.path.join(path, "OrigVsFixSSpop")
         plt.savefig(output_path, dpi=300)
         plt.close()
     else:
@@ -462,7 +544,7 @@ def plot_omega_fixed(
 
 
 def plot_imm_fixed(
-    age_per_EpS, imm_rates_orig, imm_rates_adj, E, S, output_dir=None
+    age_per_EpS, imm_rates_orig, imm_rates_adj, E, S, path=None
 ):
     """
     Plot the immigration rates implied by the data on population,
@@ -477,7 +559,7 @@ def plot_imm_fixed(
         imm_rates_adj (Numpy array): adjusted immigration rates by age
         E (int): age at which household becomes economically active
         S (int): number of years which household is economically active
-        output_dir (str): path to save figure to, if None then figure
+        path (str): path to save figure to, if None then figure
             is returned
 
     Returns:
@@ -494,8 +576,8 @@ def plot_imm_fixed(
     plt.xlim((0, E + S + 1))
     plt.legend(loc="upper center")
     # Save or return figure
-    if output_dir:
-        output_path = os.path.join(output_dir, "OrigVsAdjImm")
+    if path:
+        output_path = os.path.join(path, "OrigVsAdjImm")
         plt.savefig(output_path, dpi=300)
         plt.close()
     else:
@@ -504,13 +586,13 @@ def plot_imm_fixed(
 
 def plot_population_path(
     age_per_EpS,
-    initial_pop_pct,
     omega_path_lev,
     omega_SSfx,
-    data_year,
-    curr_year,
+    start_year,
+    year1,
+    year2,
     S,
-    output_dir=None,
+    path=None,
 ):
     """
     Plot the distribution of the population over age for various years.
@@ -518,15 +600,17 @@ def plot_population_path(
     Args:
         age_per_EpS (array_like): list of ages over which to plot
             population distribution
-        pop_2013_pct (array_like): population distribution in 2013
+        initial_pop_pct (array_like): initial year population distribution
         omega_path_lev (Numpy array): number of households by age
             over the transition path
         omega_SSfx (Numpy array): number of households by age
             in the SS
-        data_year (int): year of data for initial_pop_pct
-        curr_year (int): current year in the model
+        start_year (int): first year of data (so can get index of year1
+            and year2)
+        year1 (int): first year of data to plot
+        year2 (int): second year of data to plot
         S (int): number of years which household is economically active
-        output_dir (str): path to save figure to, if None then figure
+        path (str): path to save figure to, if None then figure
             is returned
 
     Returns:
@@ -535,23 +619,33 @@ def plot_population_path(
 
     """
     fig, ax = plt.subplots()
-    plt.plot(age_per_EpS, initial_pop_pct, label=str(data_year) + " pop.")
     plt.plot(
         age_per_EpS,
-        (omega_path_lev[:, 0] / omega_path_lev[:, 0].sum()),
-        label=str(curr_year) + " pop.",
+        (
+            omega_path_lev[start_year - year1, :]
+            / omega_path_lev[start_year - year1, :].sum()
+        ),
+        label=str(year1) + " pop.",
     )
     plt.plot(
         age_per_EpS,
         (
-            omega_path_lev[:, int(0.5 * S)]
-            / omega_path_lev[:, int(0.5 * S)].sum()
+            omega_path_lev[start_year - year2, :]
+            / omega_path_lev[start_year - year2, :].sum()
+        ),
+        label=str(year2) + " pop.",
+    )
+    plt.plot(
+        age_per_EpS,
+        (
+            omega_path_lev[int(0.5 * S), :]
+            / omega_path_lev[int(0.5 * S), :].sum()
         ),
         label="T=" + str(int(0.5 * S)) + " pop.",
     )
     plt.plot(
         age_per_EpS,
-        (omega_path_lev[:, int(S)] / omega_path_lev[:, int(S)].sum()),
+        (omega_path_lev[int(S), :] / omega_path_lev[int(S), :].sum()),
         label="T=" + str(int(S)) + " pop.",
     )
     plt.plot(age_per_EpS, omega_SSfx, label="Adj. SS pop.")
@@ -560,8 +654,8 @@ def plot_population_path(
     plt.ylabel(r"Pop. dist'n $\omega_{s}$")
     plt.legend(loc="lower left")
     # Save or return figure
-    if output_dir:
-        output_path = os.path.join(output_dir, "PopDistPath")
+    if path:
+        output_path = os.path.join(path, "PopDistPath")
         plt.savefig(output_path, dpi=300)
         plt.close()
     else:
@@ -579,7 +673,7 @@ def gen_3Dscatters_hist(df, s, t, output_dir):
             rates
         s (int): age of individual, >= 21
         t (int): year of analysis, >= 2016
-        output_dir (str): output directory for saving plot files
+        path (str): output directory for saving plot files
 
     Returns:
         None
@@ -714,7 +808,7 @@ def txfunc_graph(
         tax_func_type (str): functional form of tax functions
         params_to_plot (array_like or function): tax function parameters or
             nonparametric function
-        output_dir (str): output directory for saving plot files
+        path (str): output directory for saving plot files
 
     Returns:
         None
@@ -829,7 +923,7 @@ def txfunc_sse_plot(age_vec, sse_mat, start_year, varstr, output_dir, round):
             size is BW x S
         start_year (int): first year of budget window
         varstr (str): name of tax function being evaluated
-        output_dir (str): path to save graph to
+        path (str): path to save graph to
         round (int): which round of sweeping for outliers (0, 1, or 2)
 
     Returns:
@@ -858,7 +952,7 @@ def txfunc_sse_plot(age_vec, sse_mat, start_year, varstr, output_dir, round):
 
 
 def plot_income_data(
-    ages, abil_midp, abil_pcts, emat, output_dir=None, filesuffix=""
+    ages, abil_midp, abil_pcts, emat, t=None, path=None, filesuffix=""
 ):
     """
     This function graphs ability matrix in 3D, 2D, log, and nolog
@@ -870,46 +964,54 @@ def plot_income_data(
         abil_pcts (Numpy array): percent of population in each lifetime
             income group, length J
         emat (Numpy array): effective labor units by age and lifetime
-            income group, size SxJ
+            income group, size TxSxJ
+        t (int): model period for year, if None, then plot SS values
         filesuffix (str): suffix to be added to plot files
 
     Returns:
         None
 
     """
+    if t is None:
+        t = -1
     J = abil_midp.shape[0]
     abil_mesh, age_mesh = np.meshgrid(abil_midp, ages)
     cmap1 = matplotlib.cm.get_cmap("summer")
-    if output_dir:
+    if path:
         # Make sure that directory is created
-        utils.mkdirs(output_dir)
+        utils.mkdirs(path)
         if J == 1:
             # Plot of 2D, J=1 in levels
             plt.figure()
-            plt.plot(ages, emat)
+            plt.plot(ages, emat[t, :, :])
             filename = "ability_2D_lev" + filesuffix
-            fullpath = os.path.join(output_dir, filename)
+            fullpath = os.path.join(path, filename)
             plt.savefig(fullpath, dpi=300)
             plt.close()
 
             # Plot of 2D, J=1 in logs
             plt.figure()
-            plt.plot(ages, np.log(emat))
+            plt.plot(ages, np.log(emat[t, :, :]))
             filename = "ability_2D_log" + filesuffix
-            fullpath = os.path.join(output_dir, filename)
+            fullpath = os.path.join(path, filename)
             plt.savefig(fullpath, dpi=300)
             plt.close()
         else:
             # Plot of 3D, J>1 in levels
             fig10, ax10 = plt.subplots(subplot_kw={"projection": "3d"})
             ax10.plot_surface(
-                age_mesh, abil_mesh, emat, rstride=8, cstride=1, cmap=cmap1
+                age_mesh,
+                abil_mesh,
+                emat[t, :, :],
+                rstride=8,
+                cstride=1,
+                cmap=cmap1,
             )
             ax10.set_xlabel(r"age-$s$")
             ax10.set_ylabel(r"ability type -$j$")
             ax10.set_zlabel(r"ability $e_{j,s}$")
             filename = "ability_3D_lev" + filesuffix
-            fullpath = os.path.join(output_dir, filename)
+            fullpath = os.path.join(path, filename)
             plt.savefig(fullpath, dpi=300)
             plt.close()
 
@@ -918,7 +1020,7 @@ def plot_income_data(
             ax11.plot_surface(
                 age_mesh,
                 abil_mesh,
-                np.log(emat),
+                np.log(emat[t, :, :]),
                 rstride=8,
                 cstride=1,
                 cmap=cmap1,
@@ -927,7 +1029,7 @@ def plot_income_data(
             ax11.set_ylabel(r"ability type -$j$")
             ax11.set_zlabel(r"log ability $log(e_{j,s})$")
             filename = "ability_3D_log" + filesuffix
-            fullpath = os.path.join(output_dir, filename)
+            fullpath = os.path.join(path, filename)
             plt.savefig(fullpath, dpi=300)
             plt.close()
 
@@ -955,7 +1057,7 @@ def plot_income_data(
                     if j <= 3:
                         ax.plot(
                             ages,
-                            np.log(emat[:, j]),
+                            np.log(emat[t, :, j]),
                             label=this_label,
                             linestyle=linestyles[j],
                             color="black",
@@ -963,7 +1065,7 @@ def plot_income_data(
                     elif j > 3:
                         ax.plot(
                             ages,
-                            np.log(emat[:, j]),
+                            np.log(emat[t, :, j]),
                             label=this_label,
                             marker=markers[j - 4],
                             color="black",
@@ -975,7 +1077,7 @@ def plot_income_data(
                 ax.set_xlabel(r"age-$s$")
                 ax.set_ylabel(r"log ability $log(e_{j,s})$")
                 filename = "ability_2D_log" + filesuffix
-                fullpath = os.path.join(output_dir, filename)
+                fullpath = os.path.join(path, filename)
                 plt.savefig(fullpath, dpi=300)
                 plt.close()
     else:
@@ -1003,7 +1105,7 @@ def plot_income_data(
                 if j <= 3:
                     ax.plot(
                         ages,
-                        np.log(emat[:, j]),
+                        np.log(emat[t, :, j]),
                         label=this_label,
                         linestyle=linestyles[j],
                         color="black",
@@ -1011,7 +1113,7 @@ def plot_income_data(
                 elif j > 3:
                     ax.plot(
                         ages,
-                        np.log(emat[:, j]),
+                        np.log(emat[t, :, j]),
                         label=this_label,
                         marker=markers[j - 4],
                         color="black",
@@ -1031,6 +1133,7 @@ def plot_2D_taxfunc(
     start_year,
     tax_param_list,
     age=None,
+    E=21,  # Age at which agents become economically active in the model
     tax_func_type=["DEP"],
     rate_type="etr",
     over_labinc=True,
@@ -1086,14 +1189,17 @@ def plot_2D_taxfunc(
     if len(tax_func_type) < len(tax_param_list):
         tax_func_type = [tax_func_type[0]] * len(tax_param_list)
     for i, v in enumerate(tax_func_type):
-        assert v in ["DEP", "DEP_totalinc", "GS", "linear", "mono"]
+        assert v in ["DEP", "DEP_totalinc", "GS", "linear", "mono", "mono2D"]
     assert rate_type in ["etr", "mtrx", "mtry"]
     assert len(tax_param_list) == len(labels)
 
     # Set age and year to look at
     if age is not None:
         assert isinstance(age, int)
-        s = age - 21
+        assert age >= E
+        s = (
+            age - E
+        )  # Note: assumed age is given in E + model periods (but age below is also assumed to be calendar years)
     else:
         s = 0  # if not age-specific, all ages have the same values
     t = year - start_year
@@ -1118,7 +1224,7 @@ def plot_2D_taxfunc(
     # get tax rates for each point in the income support and plot
     fig, ax = plt.subplots()
     for i, tax_params in enumerate(tax_param_list):
-        tax_params = tax_params[rate_key][s][t]
+        tax_params = tax_params[rate_key][t][s]
         rates = txfunc.get_tax_rates(
             tax_params,
             X,
@@ -1138,9 +1244,7 @@ def plot_2D_taxfunc(
             "mtry": "mtr_capinc",
         }
         # censor data to range of the plot
-        print("OUTSIDE")
         for d, data in enumerate(data_list):
-            print("INSIDE")
             data_to_plot = data[str(year)].copy()
             if age is not None:
                 data_to_plot.drop(
@@ -1154,24 +1258,25 @@ def plot_2D_taxfunc(
             )
             # other censoring used in txfunc.py
             data_to_plot = txfunc.tax_data_sample(data_to_plot)
-            print(data_to_plot.describe())
             # set number of bins to 100 or bins of $1000 dollars
             n_bins = min(100, np.floor_divide(max_inc_amt, 1000))
             # need to compute weighted averages by group...
 
-            def wm(x):
+            def weighted_mean(x, cols, w="weight"):
                 try:
-                    np.average(x, weights=data_to_plot.loc[x.index, "weight"])
+                    return pd.Series(
+                        np.average(x[cols], weights=x[w], axis=0), cols
+                    )
                 except ZeroDivisionError:
                     return 0
 
             data_to_plot["inc_bin"] = pd.cut(data_to_plot[key1], n_bins)
-            groups = pd.DataFrame(
-                data_to_plot.groupby(["inc_bin"]).agg(
-                    rate=(rate_type_dict[rate_type], wm), income=(key1, wm)
-                )
+            groups = data_to_plot.groupby("inc_bin", observed=True).apply(
+                weighted_mean, [rate_type_dict[rate_type], key1]
+            )
+            plt.scatter(
+                groups[key1], groups[rate_type_dict[rate_type]], alpha=0.1
             )
-            plt.scatter(groups["income"], groups["rate"], alpha=0.1)
     # add legend, labels, etc to plot
     plt.legend(loc="center right")
     if title:
