@@ -1,12 +1,10 @@
 import os
 import numpy as np
 import scipy.interpolate as si
-import pkg_resources
 import paramtools
-
-# import ogcore
+import ogcore
 from ogcore import elliptical_u_est
-from ogcore.utils import rate_conversion
+from ogcore.utils import rate_conversion, extrapolate_arrays
 from ogcore.constants import BASELINE_DIR
 
 CURRENT_PATH = os.path.abspath(os.path.dirname(__file__))
@@ -35,7 +33,7 @@ class Specifications(paramtools.Parameters):
         self.num_workers = num_workers
 
         # put OG-Core version in parameters to save for reference
-        self.ogcore_version = pkg_resources.get_distribution("ogcore").version
+        self.ogcore_version = ogcore.__version__
 
         # does cheap calculations to find parameter values
         self.initialize()
@@ -121,13 +119,37 @@ class Specifications(paramtools.Parameters):
         )
 
         # set fraction of income taxes from payroll to zero initially
-        # will be updated when function tax function parameters
+        # will be updated when read in tax function parameters
         if self.T + self.S > self.BW:
             self.frac_tax_payroll = np.append(
                 self.frac_tax_payroll,
                 np.ones(self.T + self.S - self.BW) * self.frac_tax_payroll[-1],
             )
 
+        # Interpolate chi_n and create omega_SS_80 if necessary
+        if self.S < 80 and self.chi_n.shape[-1] == 80:
+            self.age_midp_80 = np.linspace(20.5, 99.5, 80)
+            reshape_chi_n = np.zeros((self.T + self.S, self.S))
+            for t in range(self.T + self.S):
+                if self.chi_n.ndim == 1:
+                    self.chi_n_interp = si.interp1d(
+                        self.age_midp_80,
+                        np.squeeze(self.chi_n),
+                        kind="cubic",
+                    )
+                else:
+                    self.chi_n_interp = si.interp1d(
+                        self.age_midp_80,
+                        np.squeeze(self.chi_n[t, :]),
+                        kind="cubic",
+                    )
+                self.newstep = 80.0 / self.S
+                self.age_midp_S = np.linspace(
+                    20 + 0.5 * self.newstep, 100 - 0.5 * self.newstep, self.S
+                )
+                reshape_chi_n[t, :] = self.chi_n_interp(self.age_midp_S)
+            self.chi_n = reshape_chi_n
+
         # Extend parameters that may vary over the time path
         tp_param_list = [
             "alpha_G",
@@ -147,24 +169,11 @@ class Specifications(paramtools.Parameters):
             "r_gov_shift",
         ]
         for item in tp_param_list:
-            this_attr = getattr(self, item)
-            if this_attr.ndim > 1:
-                this_attr = np.squeeze(this_attr, axis=1)
-            # the next if statement is a quick fix to avoid having to
-            # update all these time varying parameters if change T or S
-            # ideally, the default json values are read in again and the
-            # extension done is done again here with those defaults and
-            # the new T and S values...
-            if this_attr.size > self.T + self.S:
-                this_attr = this_attr[: self.T + self.S]
-            this_attr = np.concatenate(
-                (
-                    this_attr,
-                    np.ones((self.T + self.S - this_attr.size))
-                    * this_attr[-1],
-                )
+            param_in = getattr(self, item)
+            param_out = extrapolate_arrays(
+                param_in, dims=(self.T + self.S,), item=item
             )
-            setattr(self, item, this_attr)
+            setattr(self, item, param_out)
         # Deal with parameters that vary across industry and over time
         tp_param_list2 = [
             "Z",
@@ -173,180 +182,40 @@ class Specifications(paramtools.Parameters):
             "inv_tax_credit",
         ]
         for item in tp_param_list2:
-            this_attr = getattr(self, item)
-            if this_attr.ndim == 1:
-                # case where enter single number, so assume constant
-                # across years and industries
-                if this_attr.shape[0] == 1:
-                    this_attr = (
-                        np.ones((self.T + self.S, self.M)) * this_attr[0]
-                    )
-                # case where user enters just one year for all industries
-                if this_attr.shape[0] == self.M:
-                    this_attr = np.tile(
-                        this_attr.reshape(1, self.M), (self.T + self.S, 1)
-                    )
-                else:
-                    # case where user enters multiple years for one industry
-                    # will assume they implied values the same across industries
-                    this_attr = np.concatenate(
-                        (
-                            this_attr,
-                            np.ones((self.T + self.S - this_attr.size))
-                            * this_attr[-1],
-                        )
-                    )
-                    this_attr = np.tile(
-                        this_attr.reshape(self.T + self.S, 1), (1, self.M)
-                    )
-                this_attr = np.squeeze(this_attr, axis=2)
-            elif this_attr.ndim == 2:
-                if this_attr.shape[1] > 1 and this_attr.shape[1] != self.M:
-                    print(
-                        "please provide values of "
-                        + item
-                        + " for each industry (or one if common across "
-                        + "industries}"
-                    )
-                    assert False
-                if this_attr.shape[1] == 1:
-                    this_attr = np.tile(
-                        this_attr.reshape(this_attr.shape[0], 1), (1, self.M)
-                    )
-                if this_attr.shape[0] > self.T + self.S:
-                    this_attr = this_attr[: self.T + self.S, :]
-                this_attr = np.concatenate(
-                    (
-                        this_attr,
-                        np.ones(
-                            (
-                                self.T + self.S - this_attr.shape[0],
-                                this_attr.shape[1],
-                            )
-                        )
-                        * this_attr[-1, :],
-                    )
-                )
-            setattr(self, item, this_attr)
+            param_in = getattr(self, item)
+            param_out = extrapolate_arrays(
+                param_in, dims=(self.T + self.S, self.M), item=item
+            )
+            setattr(self, item, param_out)
         # Deal with parameters that vary across consumption good and over time
         tp_param_list3 = ["tau_c"]
         for item in tp_param_list3:
-            this_attr = getattr(self, item)
-            if this_attr.ndim == 1:
-                # case where enter single number, so assume constant
-                # across years and industries
-                if this_attr.shape[0] == 1:
-                    this_attr = (
-                        np.ones((self.T + self.S, self.I)) * this_attr[0]
-                    )
-                # case where user enters just one year for all industries
-                if this_attr.shape[0] == self.I:
-                    this_attr = np.tile(
-                        this_attr.reshape(1, self.I), (self.T + self.S, 1)
-                    )
-                else:
-                    # case where user enters multiple years for one industry
-                    # will assume they implied values the same across industries
-                    this_attr = np.concatenate(
-                        (
-                            this_attr,
-                            np.ones((self.T + self.S - this_attr.size))
-                            * this_attr[-1],
-                        )
-                    )
-                    this_attr = np.tile(
-                        this_attr.reshape(self.T + self.S, 1), (1, self.I)
-                    )
-                this_attr = np.squeeze(this_attr, axis=2)
-            elif this_attr.ndim == 2:
-                if this_attr.shape[1] > 1 and this_attr.shape[1] != self.I:
-                    print(
-                        "please provide values of "
-                        + item
-                        + " for each industry (or one if common across "
-                        + "industries"
-                    )
-                    assert False
-                if this_attr.shape[1] == 1:
-                    this_attr = np.tile(
-                        this_attr.reshape(this_attr.shape[0], 1), (1, self.I)
-                    )
-                if this_attr.shape[0] > self.T + self.S:
-                    this_attr = this_attr[: self.T + self.S, :]
-                this_attr = np.concatenate(
-                    (
-                        this_attr,
-                        np.ones(
-                            (
-                                self.T + self.S - this_attr.shape[0],
-                                this_attr.shape[1],
-                            )
-                        )
-                        * this_attr[-1, :],
-                    )
-                )
-            setattr(self, item, this_attr)
+            param_in = getattr(self, item)
+            param_out = extrapolate_arrays(
+                param_in, dims=(self.T + self.S, self.I), item=item
+            )
+            setattr(self, item, param_out)
         # Deal with parameters that vary across J and over time
         tp_param_list3 = [
             "labor_income_tax_noncompliance_rate",
             "capital_income_tax_noncompliance_rate",
         ]
         for item in tp_param_list3:
-            this_attr = getattr(self, item)
-            if this_attr.ndim == 1:
-                # case where enter single number, so assume constant
-                # across years and J
-                if this_attr.shape[0] == 1:
-                    this_attr = (
-                        np.ones((self.T + self.S, self.J)) * this_attr[0]
-                    )
-                # case where user enters just one year for all J
-                if this_attr.shape[0] == self.J:
-                    this_attr = np.tile(
-                        this_attr.reshape(1, self.J), (self.T + self.S, 1)
-                    )
-                else:
-                    # case where user enters multiple years for one J
-                    # will assume they implied values the same across J
-                    this_attr = np.concatenate(
-                        (
-                            this_attr,
-                            np.ones((self.T + self.S - this_attr.size))
-                            * this_attr[-1],
-                        )
-                    )
-                    this_attr = np.tile(
-                        this_attr.reshape(self.T + self.S, 1), (1, self.J)
-                    )
-                this_attr = np.squeeze(this_attr, axis=2)
-            elif this_attr.ndim == 2:
-                if this_attr.shape[1] > 1 and this_attr.shape[1] != self.J:
-                    print(
-                        "please provide values of "
-                        + item
-                        + " for each j (or one if common across "
-                        + "ability groups)"
-                    )
-                    assert False
-                if this_attr.shape[1] == 1:
-                    this_attr = np.tile(
-                        this_attr.reshape(this_attr.shape[0], 1), (1, self.J)
-                    )
-                if this_attr.shape[0] > self.T + self.S:
-                    this_attr = this_attr[: self.T + self.S, :]
-                this_attr = np.concatenate(
-                    (
-                        this_attr,
-                        np.ones(
-                            (
-                                self.T + self.S - this_attr.shape[0],
-                                this_attr.shape[1],
-                            )
-                        )
-                        * this_attr[-1, :],
-                    )
-                )
-            setattr(self, item, this_attr)
+            param_in = getattr(self, item)
+            param_out = extrapolate_arrays(
+                param_in, dims=(self.T + self.S, self.J), item=item
+            )
+            setattr(self, item, param_out)
+        # Deal with parameters that vary across age and over time
+        tp_param_list4 = [
+            "rho",
+        ]
+        for item in tp_param_list4:
+            param_in = getattr(self, item)
+            param_out = extrapolate_arrays(
+                param_in, dims=(self.T + self.S, self.S), item=item
+            )
+            setattr(self, item, param_out)
         # Deal with tax parameters that maybe age and time specific
         tax_params_to_TP = [
             "etr_params",
@@ -387,6 +256,11 @@ class Specifications(paramtools.Parameters):
                 if len(tax_to_set[0]) > self.S:
                     for t, v in enumerate(tax_to_set):
                         tax_to_set[t] = tax_to_set[t][: self.S]
+                if len(tax_to_set[0]) < self.S:
+                    tax_params_to_add = [tax_to_set[:][-1]] * (
+                        self.S - len(tax_to_set[0])
+                    )
+                    tax_to_set[0].extend(tax_params_to_add)
                 setattr(self, item, tax_to_set)
             else:
                 print(
@@ -400,70 +274,22 @@ class Specifications(paramtools.Parameters):
         # Try to deal with size of eta.  It may vary by S, J, T, but
         # want to allow user to enter one that varies by only S, S and J,
         # S and T, or T and S and J.
-        eta_to_set = getattr(self, "eta")
-        # this is the case that vary only by S
-        if eta_to_set.ndim == 1:
-            assert eta_to_set.shape[0] == self.S
-            eta_to_set = np.tile(
-                (
-                    np.tile(eta_to_set.reshape(self.S, 1), (1, self.J))
-                    / self.J
-                ).reshape(1, self.S, self.J),
-                (self.T + self.S, 1, 1),
-            )
-        # this could be where vary by S and J or T and S
-        elif eta_to_set.ndim == 2:
-            # case if S by J input
-            if eta_to_set.shape[0] == self.S:
-                eta_to_set = np.tile(
-                    eta_to_set.reshape(1, self.S, self.J),
-                    (self.T + self.S, 1, 1),
-                )
-                eta_to_set = eta_to_set = np.concatenate(
-                    (
-                        eta_to_set,
-                        np.tile(
-                            eta_to_set[-1, :, :].reshape(1, self.S, self.J),
-                            (self.S, 1, 1),
-                        ),
-                    ),
-                    axis=0,
-                )
-            # case if T by S input
-            elif eta_to_set.shape[0] == self.T:
-                eta_to_set = (
-                    np.tile(
-                        eta_to_set.reshape(self.T, self.S, 1), (1, 1, self.J)
-                    )
-                    / self.J
-                )
-                eta_to_set = eta_to_set = np.concatenate(
-                    (
-                        eta_to_set,
-                        np.tile(
-                            eta_to_set[-1, :, :].reshape(1, self.S, self.J),
-                            (self.S, 1, 1),
-                        ),
-                    ),
-                    axis=0,
-                )
-            else:
-                print("Eta dimensions are: ", self.eta.shape)
-                print("please give an eta that is either SxJ or TxS")
-                assert False
-        # this is the case where vary by S, J, T
-        elif eta_to_set.ndim == 3:
-            eta_to_set = eta_to_set = np.concatenate(
-                (
-                    eta_to_set,
-                    np.tile(
-                        eta_to_set[-1, :, :].reshape(1, self.S, self.J),
-                        (self.S, 1, 1),
-                    ),
-                ),
-                axis=0,
-            )
-        setattr(self, "eta", eta_to_set)
+        param_in = getattr(self, "eta")
+        param_out = extrapolate_arrays(
+            param_in, dims=(self.T + self.S, self.S, self.J), item="eta"
+        )
+        setattr(self, "eta", param_out)
+        param_in = getattr(self, "e")
+        param_out = extrapolate_arrays(
+            param_in, dims=(self.T, self.S, self.J), item="e"
+        )
+        setattr(self, "e", param_out)
+        # Extrapolate chi_n over T + S
+        param_in = getattr(self, "chi_n")
+        param_out = extrapolate_arrays(
+            param_in, dims=(self.T + self.S, self.S), item="chi_n"
+        )
+        setattr(self, "chi_n", param_out)
 
         # make sure zeta matrix sums to one (e.g., default off due to rounding)
         self.zeta = self.zeta / self.zeta.sum()
@@ -509,8 +335,9 @@ class Specifications(paramtools.Parameters):
         if self.constant_demographics:
             self.g_n_ss = 0.0
             self.g_n = np.zeros(self.T + self.S)
+            surv_rate = np.ones_like(self.rho) - self.rho
             surv_rate1 = np.ones((self.S,))  # prob start at age S
-            surv_rate1[1:] = np.cumprod(self.surv_rate[:-1], dtype=float)
+            surv_rate1[1:] = np.cumprod(surv_rate[-1, :-1], dtype=float)
             # number of each age alive at any time
             omega_SS = np.ones(self.S) * surv_rate1
             self.omega_SS = omega_SS / omega_SS.sum()
@@ -520,18 +347,6 @@ class Specifications(paramtools.Parameters):
             )
             self.omega_S_preTP = self.omega_SS
 
-        # Interpolate chi_n and create omega_SS_80 if necessary
-        if self.S < 80 and len(self.chi_n) == 80:
-            self.age_midp_80 = np.linspace(20.5, 99.5, 80)
-            self.chi_n_interp = si.interp1d(
-                self.age_midp_80, np.squeeze(self.chi_n), kind="cubic"
-            )
-            self.newstep = 80.0 / self.S
-            self.age_midp_S = np.linspace(
-                20 + 0.5 * self.newstep, 100 - 0.5 * self.newstep, self.S
-            )
-            self.chi_n = self.chi_n_interp(self.age_midp_S)
-
         # Create time series of stationarized UBI transfers
         self.ubi_nom_array = self.get_ubi_nom_objs()
 
@@ -631,8 +446,8 @@ class Specifications(paramtools.Parameters):
         """
         if not (isinstance(revision, dict) or isinstance(revision, str)):
             raise ValueError("ERROR: revision is not a dictionary or string")
-        # Skip over the adjust method if the tax paraemeters passed in
-        # are fucntions (e.g., in the case of tax_func_type = mono)
+        # Skip over the adjust method if the tax parameters passed in
+        # are functions (e.g., in the case of tax_func_type = mono)
         tax_update_dict = {}
         tax_func_params_functions = False
         try:
